# This file was auto-generated by Fern from our API Definition.

import typing

Status = typing.Union[
    typing.Literal["ready_to_upload", "uploaded", "ready", "deleted", "rejected", "error", "uploading"], typing.Any
]
