# This file was auto-generated by Fern from our API Definition.

import typing

AssetType = typing.Union[typing.Literal["file", "checkpoint", "lora", "textual_inversion", "vae", "latent"], typing.Any]
