# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations


class OctoAIEnvironment:
    PRODUCTION: OctoAIEnvironment
    SECURE_LINK: OctoAIEnvironment

    def __init__(self, *, default: str, image_gen: str, text_gen: str):
        self.default = default
        self.image_gen = image_gen
        self.text_gen = text_gen


OctoAIEnvironment.PRODUCTION = OctoAIEnvironment(
    default="https://api.octoai.cloud", image_gen="https://image.octoai.run", text_gen="https://text.octoai.run"
)
OctoAIEnvironment.SECURE_LINK = OctoAIEnvironment(
    default="https://api.securelink.octo.ai",
    image_gen="https://image.securelink.octo.ai",
    text_gen="https://text.securelink.octo.ai",
)
