# This file was auto-generated by Fern from our API Definition.

import typing

FinishReason = typing.Union[
    typing.Literal["stop", "length", "tool_calls", "content_filter", "function_call", "cancelled"], typing.Any
]
