# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .error_response_validation_errors_value import ErrorResponseValidationErrorsValue


class ErrorResponse(pydantic_v1.BaseModel):
    """
    An OpenAI API compatible schema for a error response.
    """

    code: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Optional error code.
    """

    message: str = pydantic_v1.Field()
    """
    A human-readable error message.
    """

    object: typing.Optional[typing.Any] = None
    param: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Optional error parameters.
    """

    type: str = pydantic_v1.Field()
    """
    A type identifier for the error.
    """

    validation_errors: typing.Optional[
        typing.Dict[str, typing.Optional[ErrorResponseValidationErrorsValue]]
    ] = pydantic_v1.Field(default=None)
    """
    Validation error in API.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}
