# This file was auto-generated by Fern from our API Definition.

import typing

from .function_choice import FunctionChoice

CreateChatCompletionRequestToolChoice = typing.Union[typing.Any, FunctionChoice]
